import React from 'react';
import { intersperse } from '../../utils';
import { isVoid } from '../../schema-predicates';
import { BaseView } from '../base';
import style from './type.st.css';
export const FunctionTypeView = (props) => {
    const { schema } = props;
    const restArg = schema.restArgument;
    const required = schema.requiredArguments || [];
    const returnTypeJsx = isVoid(schema.returns) ?
        'void' :
        (React.createElement(BaseView, { schemaRegistry: props.schemaRegistry, viewRegistry: props.viewRegistry, schema: schema.returns }));
    const args = schema.arguments.map((arg) => {
        const optional = required.includes(arg.name) ? '' : '?';
        const defaultValue = arg.default;
        return [
            arg.name + optional + ': ',
            (React.createElement(BaseView, { schemaRegistry: props.schemaRegistry, viewRegistry: props.viewRegistry, schema: arg })),
            defaultValue ? ` = ${defaultValue}` : ''
        ];
    });
    if (restArg) {
        args.push([
            `...${restArg.name}: `,
            (React.createElement(BaseView, { schemaRegistry: props.schemaRegistry, viewRegistry: props.viewRegistry, schema: restArg }))
        ]);
    }
    const commaSeparatedArgs = React.Children.toArray(intersperse(args, ', '));
    return (React.createElement("div", Object.assign({}, style('root', { category: 'function' }, props)),
        "(",
        commaSeparatedArgs,
        ") => ",
        returnTypeJsx));
};
//# sourceMappingURL=function.js.map